@@ -8,23 +8,26 @@ __all__ = ["Error", "DuPanClient", "DuPanShareList"]
 import errno
 
 # from base64 import b64encode
-from collections.abc import Callable, Mapping
-from functools import cached_property
+from collections.abc import deque, Callable, Mapping
+from functools import cached_property, partial
+from itertools import count
 from json import dumps, loads
 from platform import system
 from re import compile as re_compile, escape as re_escape, Pattern
 from subprocess import run
 from threading import Thread
 from typing import cast, Any, AnyStr, Final, Optional, TypedDict
-from urllib.parse import parse_qsl, urlencode, urlsplit, unquote
+from urllib.parse import parse_qsl, urlencode, urlparse, unquote
 from uuid import uuid4
 
 from ddddocr import DdddOcr # type: ignore
 from lxml.html import fromstring, tostring, HtmlElement
 from qrcode import QRCode # type: ignore
-from requests import Session, get
+from requests import get, Session
 from requests.cookies import create_cookie
 
+from .util.text import cookies_str_to_dict, text_within
+
 
 # 百度网盘 openapi 的应用，直接使用 AList 的
 # https://alist.nn.ci/guide/drivers/baidu.html
@@ -71,6 +74,21 @@ ERRNO_TO_MESSAGE: Final[dict[int, str]] = {
     9500: '你的帐号存在安全风险，已进入保护模式，请修改密码后使用，<a href="/disk/appeal" target="_blank">查看详情</a>', 
     90003: "暂无文件夹管理权限", 
 }
+SHARE_ERRORTYPE_TO_MESSAGE: Final[dict[int, str]] = {
+    0: "啊哦，你来晚了，分享的文件已经被删除了，下次要早点哟。", 
+    1: "啊哦，你来晚了，分享的文件已经被取消了，下次要早点哟。", 
+    2: "此链接分享内容暂时不可访问", 
+    3: "此链接分享内容可能因为涉及侵权、色情、反动、低俗等信息，无法访问！", 
+    5: "啊哦！链接错误没找到文件，请打开正确的分享链接!", 
+    10: "啊哦，来晚了，该分享文件已过期", 
+    11: "由于访问次数过多，该分享链接已失效", 
+    12: "因该分享含有自动备份文件夹，暂无法查看", 
+    15: "系统升级，链接暂时无法查看，升级完成后恢复正常。", 
+    17: "该链接访问范围受限，请使用正常的访问方式", 
+    123: "该链接已超过访问人数上限，可联系分享者重新分享", 
+    124: "您访问的链接已被冻结，可联系分享者进行激活", 
+    -1: "分享的文件不存在。", 
+}
 
 
 class VCodeResult(TypedDict, total=True):
@@ -78,15 +96,35 @@ class VCodeResult(TypedDict, total=True):
     vcode_str: str
 
 
-def recognize_vcode(_ocr=DdddOcr(beta=True, show_ad=False), /) -> VCodeResult:
+def decaptcha(
+    ocr: Callable[[bytes], str] = DdddOcr(beta=True, show_ad=False).classification, 
+    /, 
+    min_confirm: int = 2, 
+) -> VCodeResult:
     "识别百度网盘的验证码"
-    url = "https://pan.baidu.com/api/getcaptcha?prod=shareverify&web=1&clienttype=0&bdstoken="
+    url = "https://pan.baidu.com/api/getcaptcha?prod=shareverify&web=1&clienttype=0"
+    with get(url) as resp:
+        resp.raise_for_status()
+        data = resp.json()
+    vcode_img: str = data["vcode_img"]
+    vcode_str: str = data["vcode_str"]
+    counter: dict[str, int] = {}
     while True:
-        resp = get(url).json()
-        for i in range(10):
-            res = _ocr.classification(get(resp["vcode_img"]).content)
-            if len(res) == 4 and res.isalnum():
-                return {"vcode": res, "vcode_str": resp["vcode_str"]}
+        try:
+            with get(vcode_img, timeout=5) as resp:
+                resp.raise_for_status()
+                content = resp.content
+        except:
+            continue
+        res = ocr(content)
+        if len(res) != 4 or not res.isalnum():
+            continue
+        if min_confirm <= 1:
+            return {"vcode": res, "vcode_str": vcode_str}
+        m = counter.get(res, 0) + 1
+        if m >= min_confirm:
+            return {"vcode": res, "vcode_str": vcode_str}
+        counter[res] = m
 
 
 try:
@@ -118,56 +156,6 @@ def console_qrcode(text: str) -> None:
     qr.print_ascii(tty=True)
 
 
-def text_to_dict(
-    s: AnyStr, 
-    /, 
-    kv_sep: AnyStr | Pattern[AnyStr], 
-    entry_sep: AnyStr | Pattern[AnyStr], 
-) -> dict[AnyStr, AnyStr]:
-    if isinstance(kv_sep, (str, bytes, bytearray)):
-        search_kv_sep = re_compile(re_escape(kv_sep)).search
-    else:
-        search_kv_sep = kv_sep.search
-    if isinstance(entry_sep, (str, bytes, bytearray)):
-        search_entry_sep = re_compile(re_escape(entry_sep)).search
-    else:
-        search_entry_sep = entry_sep.search
-    d: dict[AnyStr, AnyStr] = {}
-    start = 0
-    length = len(s)
-    while start < length:
-        match = search_kv_sep(s, start)
-        if match is None:
-            break
-        l, r = match.span()
-        key = s[start:l]
-        match = search_entry_sep(s, r)
-        if match is None:
-            d[key] = s[r:]
-            break
-        l, start = match.span()
-        d[key] = s[r:l]
-    return d
-
-
-def headers_str_to_dict(
-    cookies: str, 
-    /, 
-    kv_sep: str | Pattern[str] = ": ", 
-    entry_sep: str | Pattern[str] = "\n", 
-) -> dict[str, str]:
-    return text_to_dict(cookies.strip(), kv_sep, entry_sep)
-
-
-def cookies_str_to_dict(
-    cookies: str, 
-    /, 
-    kv_sep: str | Pattern[str] = re_compile(r"\s*=\s*"), 
-    entry_sep: str | Pattern[str] = re_compile(r"\s*;\s*"), 
-) -> dict[str, str]:
-    return text_to_dict(cookies.strip(), kv_sep, entry_sep)
-
-
 class Error(OSError):
 
     @classmethod
@@ -521,21 +509,35 @@ class DuPanShareList:
             shorturl, _password = self._extract_from_url(url)
             if not password:
                 password = _password
-            # NOTE: OR url = f"https://pan.baidu.com/s/1{shorturl}"
+            # NOTE: Or use the following format, return 404 when the link is cancelled or disabled
+            #   url = f"https://pan.baidu.com/share/init?surl={shorturl}"
             if shorturl:
-                url = f"https://pan.baidu.com/share/init?surl={shorturl}"
+                url = f"https://pan.baidu.com/s/1{shorturl}"
         else:
             shorturl = url
-            url = f"https://pan.baidu.com/share/init?surl={shorturl}"
+            url = f"https://pan.baidu.com/s/1{shorturl}"
         self.url = url
         self.shorturl = shorturl
         self.password = password
         session = self.session = Session()
         session.headers["Referer"] = url
 
+    def __iter__(self, /):
+        dq = deque("/")
+        get, put = dq.popleft, dq.append
+        while dq:
+            for file in self.iterdir(get()):
+                yield file
+                if file["isdir"]:
+                    put(file["path"])
+
     @staticmethod
     def _extract_from_url(url: str, /) -> tuple[str, str]:
-        urlp = urlsplit(url)
+        urlp = urlparse(url)
+        if urlp.scheme and urlp.scheme not in ("http", "https"):
+            raise ValueError(f"url 协议只接受 'http' 和 'https'，收到 {urlp.scheme!r}，")
+        if urlp.netloc and urlp.netloc != "pan.baidu.com":
+            raise ValueError(f"url 的域名必须是 'pan.baidu.com'，收到 {urlp.netloc!r}")
         path = urlp.path
         query = dict(parse_qsl(urlp.query))
         if path == "/share/link":
@@ -544,21 +546,30 @@ class DuPanShareList:
             shorturl = query["surl"]
         elif path.startswith("/s/1"):
             shorturl = path.removeprefix("/s/1")
+        elif "/" not in path:
+            shorturl = path
         else:
-            raise ValueError("invalid share url")
-        pwd = query.get("pwd", "")
-        return shorturl, pwd
+            raise ValueError(f"invalid share url: {url!r}")
+        return shorturl, query.get("pwd", "")
 
     @staticmethod
-    def _extract_index_data(
-        content: bytes, 
-        _search=re_compile(br"locals\.mset\((.*?)\);").search, 
-        /, 
-    ) -> dict:
-        match = _search(content)
-        if match is None:
+    def _extract_indexdata(content: bytes, /) -> dict:
+        match = text_within(content, b"locals.mset(", b");")
+        if not match:
             raise OSError("没有提取到页面相关数据，可能是页面加载失败、被服务器限制访问、链接失效、分享被取消等原因")
-        return loads(match[1])
+        return loads(match)
+
+    @staticmethod
+    def _extract_yundata(
+        content: bytes, 
+        /, 
+        _sub=partial(re_compile(r"\w+(?=:)").sub, r'"\g<0>"'), 
+    ) -> Optional[dict]:
+        "从分享链接的主页中提取分享者相关的信息"
+        try:
+            return eval(_sub(text_within(content, b"window.yunData=", b";").decode("utf-8")))
+        except:
+            return None
 
     @cached_property
     def root(self, /):
@@ -580,50 +591,99 @@ class DuPanShareList:
         self.listdir_index()
         return self.__dict__["share_uk"]
 
-    def verify(self, /):
+    @cached_property
+    def yundata(self, /):
+        self.listdir_index()
+        return self.__dict__["yundata"]
+
+    def verify(self, /, use_vcode: bool = False):
         api = "https://pan.baidu.com/share/verify"
         params: dict[str, int | str] = {"surl": self.shorturl, "web": 1, "clienttype": 0}
         data = {"pwd": self.password}
-        session = self.session
+        if use_vcode:
+            data.update(cast(dict[str, str], decaptcha()))
+        post = self.session.post
         while True:
-            with session.post(api, params=params, data=data) as resp:
+            with post(api, params=params, data=data) as resp:
                 resp.raise_for_status()
                 json = resp.json()
                 errno = json["errno"]
                 if not errno:
                     break
                 if errno == -62:
-                    data.update(cast(dict[str, str], recognize_vcode()))
+                    data.update(cast(dict[str, str], decaptcha()))
                 else:
                     raise OSError(json)
 
-    def listdir_index(self, /) -> dict:
+    def iterdir(self, /, dir="/"):
+        if dir in ("", "/"):
+            yield from self.listdir_index()
+            return
+        if not hasattr(self, "share_uk"):
+            self.listdir_index()
+        if not dir.startswith("/"):
+            dir = self.root + "/" + dir
+        api = "https://pan.baidu.com/share/list"
+        params = {
+            "uk": self.share_uk, 
+            "shareid": self.share_id, 
+            "order": "other", 
+            "desc": 1, 
+            "showempty": 0, 
+            "clienttype": 0, 
+            "web": 1, 
+            "page": 1, 
+            "num": 100, 
+            "dir": dir, 
+        }
+        get = self.session.get
+        while True:
+            ls = Error.check(get(api, params=params).json())["list"]
+            yield from ls
+            if len(ls) < 100:
+                return
+            params["page"] += 1
+
+    def listdir_index(self, /, try_times: int = 5) -> dict:
         url = self.url
         password = self.password
         session = self.session
-        while True:
+        if try_times <= 0:
+            it = count()
+        else:
+            it = range(try_times)
+        for _ in it:
             with session.get(url) as resp:
                 resp.raise_for_status()
                 content = resp.content
-                data = self._extract_index_data(content)
+                data = self._extract_indexdata(content)
                 if b'"verify-form"' in content:
                     if not password:
                         raise OSError("需要密码")
-                    self.verify()
+                    self.verify(b'"change-code"' in content)
                 else:
-                    file_list = data["file_list"]
-                    if file_list is None:
-                        raise OSError("没有找到下载文件，可能是链接失效、分享被取消等原因")
+                    if data["errno"]:
+                        data["errno_reason"] = ERRNO_TO_MESSAGE.get(data["errno"])
+                        data["errortype_reason"] = SHARE_ERRORTYPE_TO_MESSAGE.get(data.get("errortype", -1))
+                        raise OSError(data)
+                    file_list = data.get("file_list")
+                    if not file_list:
+                        raise OSError("无下载文件，可能是链接失效、分享被取消、删除了所有分享文件等原因")
+                    self.yundata = self._extract_yundata(content)
                     if file_list:
-                        root = file_list[0]["path"].rsplit("/", 1)[0]
+                        root = root2 = file_list[0]["path"].rsplit("/", 1)[0]
+                        if len(file_list) > 1:
+                            root2 = file_list[1]["path"].rsplit("/", 1)[0]
                     else:
-                        root = "/"
+                        root = root2 = "/"
                     self.__dict__.update(
                         root = root, 
+                        root2 = root2, 
                         share_uk = data["share_uk"], 
                         share_id = data["shareid"], 
                     )
                     return file_list
+        raise RuntimeError("too many attempts")
 
     def listdir(self, dir="/", page=1, num=0):
         if dir in ("", "/"):
@@ -662,4 +722,5 @@ class DuPanShareList:
 
 
 # TODO: 上传下载使用百度网盘的openapi，直接使用 alist 已经授权的 token
+# TODO: 百度网盘转存时，需要保持相对路径
 
